<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" 
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:xalan="http://xml.apache.org/xalan"
				xmlns:dyn="http://exslt.org/dynamic"
				exclude-result-prefixes="xalan dyn">
	
	<xsl:output encoding="UTF-8" method="xml" indent="yes" />
	
	<xsl:param name="pRolesMappingFile"/>
	
	<xsl:variable name="vRolesMappingXml" select="document($pRolesMappingFile)"/>
	
	<!-- Niveau USER -->
	<xsl:template match="importUsers/createOrUpdate/a/o[s[@k='userType']='user']" name="tUserRoles" priority="2">
		<xsl:param name="pCurrentLevel" select="'user'"/>
		<xsl:copy>
			<xsl:apply-templates select="@*|node()[not(@k='grantedRoles') and not(@k='refusedRoles')]"/>
			<!-- grantedRoles -->
			<xsl:choose>
				<xsl:when test="a[@k='grantedRoles']">
					<xsl:apply-templates select="a[@k='grantedRoles']"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="vAddRoleList">
					<xsl:for-each select="$vRolesMappingXml//addRole[onlyIfNoRole/@type='allow']">
						<xsl:if test="(not(restrictOnScope) or restrictOnScope/@level=$pCurrentLevel)">
							<xsl:copy-of select="."/>
						</xsl:if>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="vAddRoleListNS" select="xalan:nodeset($vAddRoleList)"/>
				<xsl:if test="$vAddRoleListNS/*">
					<a k="grantedRoles">
						<xsl:for-each select="$vAddRoleListNS/addRole">
							<s>
								<xsl:value-of select="@name"/>
							</s>
						</xsl:for-each>
					</a>
				</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
			<!-- refusedRoles -->
			<xsl:choose>
				<xsl:when test="a[@k='refusedRoles']">
					<xsl:apply-templates select="a[@k='refusedRoles']"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="vAddRoleList">
					<xsl:for-each select="$vRolesMappingXml//addRole[onlyIfNoRole/@type='deny']">
						<xsl:if test="(not(restrictOnScope) or restrictOnScope/@level=$pCurrentLevel)">
							<xsl:copy-of select="."/>
						</xsl:if>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="vAddRoleListNS" select="xalan:nodeset($vAddRoleList)"/>
				<xsl:if test="$vAddRoleListNS/*">
					<a k="refusedRoles">
						<xsl:for-each select="$vAddRoleListNS/addRole">
							<s>
								<xsl:value-of select="@name"/>
							</s>
						</xsl:for-each>
					</a>
				</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="importUsers/createOrUpdate/a/o[s[@k='userType']='user']/a[contains(@k, 'Roles')]" name="tUserRolesList" priority="2">
		<xsl:param name="pCurrentLevel" select="'user'"/>
		<xsl:copy>
			<xsl:apply-templates select="@*"/>
			<!-- # Renames rôles -->
			<xsl:variable name="vRolesNs">
				<xsl:for-each select="s">
					<xsl:variable name="vRenameRoleList">
						<xsl:for-each select="$vRolesMappingXml//renameRole">
							<xsl:if test="not(restrictOnScope) or restrictOnScope/@level=$pCurrentLevel">
								<xsl:copy-of select="."/>
							</xsl:if>
						</xsl:for-each>
					</xsl:variable>
					<xsl:variable name="vRenameRoleListNS" select="xalan:nodeset($vRenameRoleList)"/>
					<xsl:variable name="vToRolesNode" select="$vRenameRoleListNS/renameRole[@from=current()/text()]"/>
					<xsl:choose>
						<!-- Renommage -->
						<xsl:when test="$vToRolesNode/@to">
							<xsl:for-each select="xalan:tokenize($vToRolesNode/@to)">
								<s>
									<xsl:value-of select="."/>
								</s>
							</xsl:for-each>
						</xsl:when>
						<!-- Suppression -->
						<xsl:when test="$vToRolesNode"/>
						<!-- Inchangé -->
						<xsl:otherwise>
							<s>
								<xsl:value-of select="."/>
							</s>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:variable>
			<xsl:copy-of select="$vRolesNs"/>
			<!-- # addRoles -->
			<xsl:variable name="vRolesType">
				<xsl:choose><xsl:when test="@k='grantedRoles'">allow</xsl:when><xsl:when test="@k='refusedRoles'">deny</xsl:when></xsl:choose>
			</xsl:variable>
			<xsl:variable name="vHasRole" select="normalize-space($vRolesNs)"/>
			<xsl:variable name="vAddRoleList">
				<xsl:for-each select="$vRolesMappingXml//addRole[@type=$vRolesType]">
					<xsl:if test="(not(restrictOnScope) or restrictOnScope/@level=$pCurrentLevel) and (not(onlyIfNoRole) or (not($vHasRole) and onlyIfNoRole/@type=$vRolesType))">
						<xsl:copy-of select="."/>
					</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="vAddRoleListNS" select="xalan:nodeset($vAddRoleList)"/>
			<xsl:for-each select="$vAddRoleListNS/addRole">
				<s>
					<xsl:value-of select="@name"/>
				</s>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	
	<!-- Niveau GROUP -->
	<xsl:template match="importUsers/createOrUpdate/a/o[s[@k='userType']='group']" priority="2">
		<xsl:call-template name="tUserRoles">
			<xsl:with-param name="pCurrentLevel" select="'group'"/>
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="importUsers/createOrUpdate/a/o[s[@k='userType']='group']/a[contains(@k, 'Roles')]" priority="2">
		<xsl:call-template name="tUserRolesList">
			<xsl:with-param name="pCurrentLevel" select="'group'"/>
		</xsl:call-template>
	</xsl:template>
	
	<!-- Niveau WSP et SPACE -->
	<xsl:template match="createWsp" priority="2">
		<xsl:apply-templates select="descendant::specifyRoles"/>
	</xsl:template>
	<xsl:template match="specifyRoles" priority="2">
		<xsl:variable name="vWspCode" select="ancestor::createWsp/@varCreatedWspCode"/>
		<xsl:variable name="vSrcUri">
			<xsl:choose>
				<!-- item/espace -->
				<xsl:when test="../@srcUri">
					<xsl:value-of select="../@srcUri"/>
				</xsl:when>
				<!-- wsp -->
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="vCurrentLevel">
			<xsl:choose>
				<!-- espace -->
				<xsl:when test="string($vSrcUri)">space</xsl:when>
				<!-- wsp -->
				<xsl:otherwise>wsp</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<specifyRoles wspCode="{$vWspCode}" srcUri="{$vSrcUri}">
			<xsl:for-each select="*">
				<xsl:copy>
					<xsl:apply-templates select="@*[name()!='roles']"/>
					<xsl:attribute name="roles">
						<!-- # Renames rôles -->
						<xsl:for-each select="xalan:tokenize(@roles)">
							<xsl:variable name="vRenameRoleList">
								<xsl:for-each select="$vRolesMappingXml//renameRole">
									<xsl:if test="not(restrictOnScope) or restrictOnScope/@level=$vCurrentLevel">
										<xsl:copy-of select="."/>
									</xsl:if>
								</xsl:for-each>
							</xsl:variable>
							<xsl:variable name="vRenameRoleListNS" select="xalan:nodeset($vRenameRoleList)"/>
							<xsl:variable name="vToRolesNode" select="$vRenameRoleListNS/renameRole[@from=current()]"/>
							<xsl:choose>
								<!-- Renommage -->
								<xsl:when test="$vToRolesNode/@to">
									<xsl:for-each select="xalan:tokenize($vToRolesNode/@to)">
										<xsl:value-of select="."/>
										<xsl:text> </xsl:text>
									</xsl:for-each>
								</xsl:when>
								<!-- Suppression -->
								<xsl:when test="$vToRolesNode"/>
								<!-- Inchangé -->
								<xsl:otherwise>
									<xsl:value-of select="."/>
									<xsl:text> </xsl:text>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:for-each>
						<!-- # addRoles -->
						<!--
						<xsl:variable name="vRolesType">
							<xsl:choose><xsl:when test="name()='allowRoles'">allow</xsl:when><xsl:when test="name()='denyRoles'">deny</xsl:when></xsl:choose>
						</xsl:variable>
						<xsl:variable name="vAddRoleList">
							<xsl:for-each select="$vRolesMappingXml//addRole[not(restrictOnScope)][@type=$vRolesType]">
								<xsl:if test="not(restrictOnScope) or restrictOnScope/@level=$vCurrentLevel">
									<xsl:copy-of select="."/>
								</xsl:if>
							</xsl:for-each>
						</xsl:variable>
						<xsl:variable name="vAddRoleListNS" select="xalan:nodeset($vAddRoleList)"/>
						<xsl:for-each select="$vAddRoleListNS/addRole">
							<xsl:value-of select="@name"/>
							<xsl:text> </xsl:text>
						</xsl:for-each>
						-->
					</xsl:attribute>
				</xsl:copy>
			</xsl:for-each>
		</specifyRoles> 

	</xsl:template>
	
	<!-- Champs non repris -->
	<xsl:template match="s[@k='authMethod'] | n[@k='pwdDt'] | s[@k='digestPwd'] | s[@k='groupName'] | n[@k='userCreationDt'] | a[@k='groups'] | s[@k='lastName'] | s[@k='firstName'] | s[@k='email']"/>
	
	<!-- ### -->
	<!-- # On recopie tout par défaut -->
	<!-- # -->
	<xsl:template match="@*|node()">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()"/>
		</xsl:copy>
	</xsl:template>
	
</xsl:stylesheet>